import csv
import codecs
import numpy as np
from sklearn import svm, linear_model
from sklearn.metrics import f1_score, precision_score, recall_score
from sklearn.model_selection import cross_val_score, train_test_split
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.gaussian_process import GaussianProcessClassifier
from sklearn.gaussian_process.kernels import DotProduct, RBF, WhiteKernel
from sklearn.neighbors import KNeighborsClassifier
from sklearn.neural_network import MLPClassifier
from sklearn.linear_model import RidgeClassifier


def loadDataset(filename):
    dataset = []
    csv_reader = csv.reader(codecs.open('data/' + filename + '.csv', 'rU', 'utf-8'), delimiter=',')
    for row in csv_reader:
        dataset.append(row)
    return dataset



def combineAllTweetsPerUser(dataset):
    users = {}
    for row in dataset:
        TWETINDEX = 5 if len(row) == 7 else 7
        username = row[1]
        if username in users:
            users[username][5] += row[TWETINDEX]
        else:
            users[username] = row[0:5] + [row[5], row[6]]
    return list(users.values())


np.random.seed(0)
data = loadDataset('train')
users = combineAllTweetsPerUser(data)
np.random.shuffle(users)
del data

test_data = loadDataset('test')
test_users = combineAllTweetsPerUser(test_data)
del test_data

labels = {'unrelated': 0, 'pro_ed': 1, 'pro_recovery': 0}


y = [labels[u[6]] for u in users]

tweet_corpus = [u[5] for u in users]


X_train, X_test, y_train, y_test = train_test_split(tweet_corpus, y, test_size=0.2)



def getScores(x, y, C):
    clf = svm.LinearSVC(C=C)
    f1_score = cross_val_score(clf, x, y, cv=5, scoring='f1')
    f1_mean = np.mean(f1_score)
    print('f1', f1_mean)
    precision = cross_val_score(clf, x, y, cv=5, scoring='precision')
    p_mean = np.mean(precision)
    print('precision', p_mean)
    recall = cross_val_score(clf, x, y, cv=5, scoring='recall')
    r_mean = np.mean(recall)
    print('recall', r_mean)
    return round(p_mean, 3), round(r_mean, 3), round(f1_mean, 3)


uni_vectorizer = TfidfVectorizer(ngram_range=(1, 1), stop_words='english', max_features=20000)
uni_x = uni_vectorizer.fit_transform(np.array(X_train))
uni_x_test = uni_vectorizer.transform(np.array(X_test))
print('unigrams')


def create_table(matrix):
    table_string = ''
    for row in matrix:
        for element in row[:-1]:
            table_string += str(element) + ' & '
        table_string += str(row[-1]) + ' \\\\ \n'
    return table_string


latex_table = []
for C in [0.01, 0.1, 0.5, 1, 1.5, 2, 3, 5, 7, 10]:
    latex_table.append([C] + list(getScores(uni_x, y_train, C)))



print(create_table(latex_table))


kernels = ['rbf', 'linear', 'sigmoid'] #'poly' funker ikke

latex_table = []
for kernel in kernels:
    clf = svm.SVC(kernel=kernel, gamma='scale')
    clf.fit(uni_x, y_train)
    y_pred = clf.predict(uni_x_test)
    f1 = f1_score(y_test, y_pred, average='binary')
    p = precision_score(y_test, y_pred, average='binary')
    r = recall_score(y_test, y_pred, average='binary')
    print(f1, p, r)
    latex_table.append([kernel, round(p, 3), round(r, 3), round(f1, 3)])



print(create_table(latex_table))



latex_table.append(["KNN"])
for n in [1, 2, 5, 8, 10, 15]:
    clf = KNeighborsClassifier(n_neighbors=n, metric='hamming')
    clf.fit(uni_x.toarray(), y_train)
    y_pred = clf.predict(uni_x_test.toarray())
    f1 = f1_score(y_test, y_pred, average='binary')
    p = precision_score(y_test, y_pred, average='binary')
    r = recall_score(y_test, y_pred, average='binary')
    print(f1, p, r)
    latex_table.append([n, round(p, 3), round(r, 3), round(f1, 3)])


print(create_table(latex_table))




latex_table.append(["MLP"])
for layerSize in [50, 100, 200, 500]:
    clf = MLPClassifier(hidden_layer_sizes=(50, ))
    clf.fit(uni_x, y_train)
    y_pred = clf.predict(uni_x_test)
    f1 = f1_score(y_test, y_pred, average='binary')
    p = precision_score(y_test, y_pred, average='binary')
    r = recall_score(y_test, y_pred, average='binary')
    print(f1, p, r)
    latex_table.append([layerSize, round(p, 3), round(r, 3), round(f1, 3)])


print(create_table(latex_table))


latex_table.append(["RC"])
for alpha in [0.001, 0.01, 0.1, 1, 2, 4, 6]:
    clf = RidgeClassifier(alpha=alpha)
    clf.fit(uni_x, y_train)
    y_pred = clf.predict(uni_x_test)
    f1 = f1_score(y_test, y_pred, average='binary')
    p = precision_score(y_test, y_pred, average='binary')
    r = recall_score(y_test, y_pred, average='binary')
    print(f1, p, r)
    latex_table.append([alpha, round(p, 3), round(r, 3), round(f1, 3)])


print(create_table(latex_table))


gp_kernels = [DotProduct() + WhiteKernel(), RBF()]


latex_table.append(["GP"])
for kernel in gp_kernels:
    clf = GaussianProcessClassifier(kernel=kernel)
    clf.fit(uni_x.toarray(), y_train)
    y_pred = clf.predict(uni_x_test.toarray())
    f1 = f1_score(y_test, y_pred, average='binary')
    p = precision_score(y_test, y_pred, average='binary')
    r = recall_score(y_test, y_pred, average='binary')
    print(f1, p, r)
    latex_table.append([kernel, round(p, 3), round(r, 3), round(f1, 3)])


print(create_table(latex_table))

