# -*- coding: utf-8 -*-
import gensim
import numpy as np
import pandas as pd
import csv
import codecs
from sklearn import svm
from langdetect import detect
from sklearn.feature_extraction.text import TfidfVectorizer
from keras.preprocessing.text import Tokenizer
from keras.preprocessing.sequence import pad_sequences
from sklearn.gaussian_process import GaussianProcessRegressor
from sklearn.gaussian_process.kernels import DotProduct, WhiteKernel
import re
from nltk.stem import WordNetLemmatizer, SnowballStemmer
from nltk.stem.porter import *
from scipy.sparse import hstack
from sklearn.neural_network import MLPClassifier
from sklearn.metrics import f1_score, precision_score, recall_score

##########################################################################
##########################################################################
###### Preprocess Personality
##########################################################################
##########################################################################

def preprocess_twitterPersonality(text):
    # lowercase
    text = ''.join(text).lower()
    # mentions & URLs
    text = ' MENTION '.join(re.split(r'\*propname\*', text))
    text = ' URL '.join(re.split(r'http:////[a-z0-9/._-]*', text))
    # remove symbols
    text = ' '.join(re.split(r'\W+', text))
    return text



mypersonality_raw = pd.read_csv('personality/mypersonality.csv')

# remove unneeded fields
relevantFields = mypersonality_raw[['#AUTHID', 'STATUS', 'sEXT', 'sNEU', 'sAGR', 'sCON', 'sOPN']]

# combine the users
mypersonality_combined_tweets = relevantFields.groupby(['#AUTHID', 'sEXT', 'sNEU', 'sAGR', 'sCON', 'sOPN'])['STATUS'].apply(lambda x: ' '.join(x.astype(str))).reset_index()


for index, row in mypersonality_combined_tweets.iterrows():
    row['STATUS'] = preprocess_twitterPersonality(row['STATUS'])
    mypersonality_combined_tweets.loc[index, 'STATUS'] = row['STATUS']
    # checking for english
    if detect(row["STATUS"]) != 'en':
        print(row["STATUS"])
        mypersonality_combined_tweets.drop(mypersonality_combined_tweets.index[index])


mypersonality_df = mypersonality_combined_tweets[['sEXT', 'sNEU', 'sAGR', 'sCON', 'sOPN', 'STATUS']]
mypersonality_df.columns = ['sEXT', 'sNEU', 'sAGR', 'sCON', 'sOPN', 'TEXT']

ext_y = []
ext_n = []
agr_y = []
agr_n = []
neu_y = []
neu_n = []
con_y = []
con_n = []
opn_y = []
opn_n = []


for index, row in mypersonality_raw.iterrows():
    if row["cEXT"] == 'y':
        ext_y.append(row["sEXT"])
    else:
        ext_n.append(row["sEXT"])
    if row["cAGR"] == 'y':
        agr_y.append(row["sAGR"])
    else:
        agr_n.append(row["sAGR"])
    if row["cNEU"] == 'y':
        neu_y.append(row["sNEU"])
    else:
        neu_n.append(row["sNEU"])
    if row["cCON"] == 'y':
        con_y.append(row["sCON"])
    else:
        con_n.append(row["sCON"])
    if row["cOPN"] == 'y':
        opn_y.append(row["sOPN"])
    else:
        opn_n.append(row["sOPN"])


# average labels based on twitter data
numericlabels_ext = {'y': np.mean(ext_y), 'n': np.mean(ext_n)}
numericlabels_agr = {'y': np.mean(agr_y), 'n': np.mean(agr_n)}
numericlabels_neu = {'y': np.mean(neu_y), 'n': np.mean(neu_n)}
numericlabels_con = {'y': np.mean(con_y), 'n': np.mean(con_n)}
numericlabels_opn = {'y': np.mean(opn_y), 'n': np.mean(opn_n)}


def preprocess_essays(text):
    # lowercase
    text = ''.join(text).lower()
    # remove symbols
    text = ' '.join(re.split(r'\W+', text))
    return text


essays_raw = pd.read_csv('personality/essays2.csv', delimiter=',',quotechar='"')

# remove unneeded fields
essays_df = essays_raw[['TEXT', 'cEXT', 'cNEU', 'cAGR', 'cCON', 'cOPN']]

for index, row in essays_df.iterrows():
    row["TEXT"] = preprocess_essays(row["TEXT"])
    row["cEXT"] = numericlabels_ext[row["cEXT"]]
    row["cNEU"] = numericlabels_agr[row["cNEU"]]
    row["cAGR"] = numericlabels_neu[row["cAGR"]]
    row["cCON"] = numericlabels_con[row["cCON"]]
    row["cOPN"] = numericlabels_opn[row["cOPN"]]


essays_df.columns = ['TEXT', 'sEXT', 'sNEU', 'sAGR', 'sCON', 'sOPN']

# Combining the two datasets
train_personality = pd.concat([essays_df, mypersonality_df], ignore_index=True)
del essays_df
del essays_raw
del mypersonality_df
del mypersonality_raw

##########################################################################
##########################################################################
###### GLOVE
##########################################################################
##########################################################################


NB_WORDS = 10000  # Number of words to put in the dictionary
GLOVE_DIM = 200  # Dimensions of the GloVe embeddings (25, 50, 100 or 200)

X_train_personality = [gensim.parsing.preprocessing.remove_stopwords(text) for text
           in train_personality['TEXT'].values.tolist()]

# Using keras tokenizer
tk = Tokenizer(num_words=NB_WORDS,
               filters='!"#$%&()*+,-./:;<=>?@[\]^_`{"}~\t\n',
               lower=True,
               split=" ")
tk.fit_on_texts(X_train_personality)
X_train_personality_seq = tk.texts_to_sequences(X_train_personality)
X_train_personality_seq_trunc = pad_sequences(X_train_personality_seq, maxlen=677)


glove_file = 'glove.twitter.27B.' + str(GLOVE_DIM) + 'd.txt'
# This is the dictionary that will hold the pretrain_personalityed values from the model
emb_dict = {}
glove = open('personality/glove.twitter.27B/' + glove_file)
for line in glove:
    values = line.split()
    word = values[0]
    vector = np.asarray(values[1:], dtype='float32')
    emb_dict[word] = vector


glove.close()


emb_matrix = np.zeros((NB_WORDS, GLOVE_DIM))
for w, i in tk.word_index.items():
    if i < NB_WORDS:
        vect = emb_dict.get(w)
        if vect is not None:
            emb_matrix[i] = vect
    else:
        break


# Average embedding as a feature for the entire doc
def document_avg_embedding(dataset):
    emb_avg = []
    # Averaging the word vectors for each text
    for vlog_index in range(len(dataset)):
        emb_total = np.zeros(GLOVE_DIM)
        for word_index in range(len(dataset[vlog_index])):
            emb_total = np.add(emb_total, emb_matrix[dataset[vlog_index, word_index]])
        emb_avg.append(emb_total / np.sqrt(emb_total.dot(emb_total)))
    return emb_avg


emb_avg = document_avg_embedding(X_train_personality_seq_trunc)


y_train_personality = [pd.to_numeric(train_personality.sEXT),
                       pd.to_numeric(train_personality.sAGR),
                       pd.to_numeric(train_personality.sCON),
                       pd.to_numeric(train_personality.sNEU),
                       pd.to_numeric(train_personality.sOPN)]


def buld_model(x_train, y_train):
    clf = svm.LinearSVR().fit(x_train, y_train)
    return clf


personality_models = []

for i in range(len(y_train_personality)):
    # Appending a classifier for each of the personality traits to the list
    personality_models.append(buld_model(emb_avg, y_train_personality[i]))

del emb_avg
##########################################################################
##########################################################################
###### PRO-ED
##########################################################################
##########################################################################


def loadDataset(filename):
    dataset = []
    csv_reader = csv.reader(codecs.open('data/' + filename + '.csv', 'rU', 'utf-8'), delimiter=',')
    for row in csv_reader:
        dataset.append(row)
    return dataset


def combineAllTweetsPerUser(dataset):
    users = {}
    for row in dataset:
        TWETINDEX = 5 if len(row) == 7 else 7
        username = row[1]
        if username in users:
            users[username][5] += row[TWETINDEX] + " "
        else:
            users[username] = row[0:5] + [row[5], row[6]]
    return list(users.values())


np.random.seed(0)
data = loadDataset('train')
users = combineAllTweetsPerUser(data)
del data

data_test = loadDataset('test')
users_test = combineAllTweetsPerUser(data_test)
del data_test

labels = {'unrelated': 0, 'pro_ed': 1, 'pro_recovery': 0}
y_train = [labels[u[6]] for u in users]
tweet_corpus = [u[5] for u in users]

y_test = [labels[u[6]] for u in users_test]
tweet_corpus_test = [u[5] for u in users_test]

##########################################################################
###### Personality feature
##########################################################################
print("starting personality detection")

X_train_glove = [gensim.parsing.preprocessing.remove_stopwords(user) for user
                 in tweet_corpus]
X_test_glove = [gensim.parsing.preprocessing.remove_stopwords(user) for user
                in tweet_corpus_test]


# Using keras tokenizer
tk = Tokenizer(num_words=NB_WORDS,
               filters='!"#$%&()*+,-./:;<=>?@[\]^_`{"}~\t\n',
               lower=True,
               split=" ")
tk.fit_on_texts(X_train_glove)
X_train_glove_seq = tk.texts_to_sequences(X_train_glove)
X_train_glove_seq_trunc = pad_sequences(X_train_glove_seq, maxlen=677)

X_test_glove_seq = tk.texts_to_sequences(X_test_glove)
X_test_glove_seq_trunc = pad_sequences(X_test_glove_seq, maxlen=677)


emb_avg_pro_ed = document_avg_embedding(X_train_glove_seq_trunc)
emb_avg_pro_ed_test = document_avg_embedding(X_test_glove_seq_trunc)

personality_x = []
for clf in personality_models:
    personality_x.append(clf.predict(emb_avg_pro_ed))

personality_x_test = []
for clf in personality_models:
    personality_x_test.append(clf.predict(emb_avg_pro_ed_test))


personality_x_recomp = []
for i in range(len(personality_x[0])):
    personality_x_recomp.append([personality_x[j][i] for j in range(5)])


personality_x_test_recomp = []
for i in range(len(personality_x_test[0])):
    personality_x_test_recomp.append([personality_x_test[j][i] for j in range(5)])


del personality_x
del personality_x_test
##########################################################################
###### n-grams
##########################################################################

print('unigrams')
uni_vectorizer = TfidfVectorizer(ngram_range=(1, 1), stop_words='english', max_features=1200)
uni_x = uni_vectorizer.fit_transform(tweet_corpus)
uni_x_test = uni_vectorizer.transform(tweet_corpus_test)
del uni_vectorizer

print('bigrams')
bi_vectorizer = TfidfVectorizer(ngram_range=(2, 2), stop_words='english', max_features=5000)
bi_x = bi_vectorizer.fit_transform(tweet_corpus)
bi_x_test = bi_vectorizer.transform(tweet_corpus_test)
del bi_vectorizer
##########################################################################
###### Topic models
##########################################################################
print("starting Topic models")
def removeStopWords(tokens):
    filtered_sentence = [w for w in tokens if w not in gensim.parsing.preprocessing.STOPWORDS]
    return filtered_sentence


def lemmatize_stemming(tokens):
    stemmer = SnowballStemmer("english")
    newTokens = []
    for w in tokens:
        newTokens.append(stemmer.stem(WordNetLemmatizer().lemmatize(w, pos='v')))
    return newTokens


def preprocess_tm(text):
    word_tokens = gensim.utils.simple_preprocess(text)
    text_stopwords_removed = removeStopWords(word_tokens)
    lemmatizedStemmed = lemmatize_stemming(text_stopwords_removed)
    return lemmatizedStemmed


def createFeatureset(lda, corpus):
    featureset = []
    for i in range(len(corpus)):
        featureset.append(lda[corpus[i]])
    X = []
    for i in range(len(featureset)):
        row = featureset[i]
        X.append([0]*15)
        for a, b in row:
            X[i][a] = b
    X = np.array(X)
    return X


processed_docs = []
count = 0
for user in tweet_corpus:
    count += 1
    processed_docs.append(preprocess_tm(user))
    if count % 500 == 0:
        count


processed_docs_test = []
count = 0
for user in tweet_corpus_test:
    count += 1
    processed_docs_test.append(preprocess_tm(user))
    if count % 500 == 0:
        count


dictionary = gensim.corpora.Dictionary(processed_docs)
dictionary.filter_extremes(no_below=20, no_above=0.5, keep_n=10000)
bow_corpus = [dictionary.doc2bow(doc) for doc in processed_docs]
bow_corpus_test = [dictionary.doc2bow(doc) for doc in processed_docs_test]
len(bow_corpus_test)

del processed_docs
del processed_docs_test
lda_model = gensim.models.LdaMulticore(bow_corpus, num_topics=10, id2word=dictionary, passes=2, workers=2)
tm_x = createFeatureset(lda_model, bow_corpus)
tm_x_test = createFeatureset(lda_model, bow_corpus_test)
del dictionary
del lda_model
del bow_corpus
del bow_corpus_test
##########################################################################
##########################################################################
###### PRO-ED Final classifiers
##########################################################################
##########################################################################
print("starting to build final classifiers")
def create_table(matrix):
    table_string = ''
    for row in matrix:
        for element in row[:-1]:
            table_string += str(element) + ' & '
        table_string += str(row[-1]) + ' \\\\ \n'
    return table_string


combined_x_with_personality = hstack([personality_x_recomp, uni_x, bi_x, tm_x])
combined_x_with_personality_test = hstack([personality_x_test_recomp, uni_x_test, bi_x_test, tm_x_test])

combined_x = hstack([uni_x, bi_x, tm_x])
combined_x_test = hstack([uni_x_test, bi_x_test, tm_x_test])


featuresets = [(combined_x_with_personality, combined_x_with_personality_test, "with personality"), (combined_x, combined_x_test, "without personality")]
#featuresets = [(personality_x_recomp, personality_x_test_recomp, "personality"), (uni_x, uni_x_test, "uni"), (bi_x, bi_x_test, "bi"), (tm_x, tm_x_test, "tm")]

latex_table = []

sum_f1_mlp = [0]*len(featuresets)
sum_p_mlp = [0]*len(featuresets)
sum_r_mlp = [0]*len(featuresets)

sum_f1_svm = [0]*len(featuresets)
sum_p_svm = [0]*len(featuresets)
sum_r_svm = [0]*len(featuresets)

for _ in range(5):
    for i in range(len(featuresets)):
        featureset = featuresets[i]
        clf_mlp = MLPClassifier(hidden_layer_sizes=(50, )).fit(featureset[0], y_train)
        y_pred_mlp = clf_mlp.predict(featureset[1])
        sum_f1_mlp[i] += f1_score(y_test, y_pred_mlp, average='weighted')
        sum_p_mlp[i] += precision_score(y_test, y_pred_mlp, average='weighted')
        sum_r_mlp[i] += recall_score(y_test, y_pred_mlp, average='weighted')
        clf_svm = svm.LinearSVC(C=2).fit(featureset[0], y_train)
        y_pred_svm = clf_svm.predict(featureset[1])
        sum_f1_svm[i] += f1_score(y_test, y_pred_svm, average='weighted')
        sum_p_svm[i] += precision_score(y_test, y_pred_svm, average='weighted')
        sum_r_svm[i] += recall_score(y_test, y_pred_svm, average='weighted')
    print(create_table(latex_table))


for i in range(len(sum_f1_mlp)):
    latex_table.append(["MLP", featuresets[i][2], round(sum_p_mlp[i]/5, 3), round(sum_r_mlp[i]/5, 3), round(sum_f1_mlp[i]/5, 3)])
    latex_table.append(["SVM", featuresets[i][2], round(sum_p_svm[i]/5, 3), round(sum_r_svm[i]/5, 3), round(sum_f1_svm[i]/5, 3)])


print(create_table(latex_table))
