import csv
import codecs
import numpy as np
from sklearn import svm
from sklearn.model_selection import cross_val_score, cross_val_predict
from sklearn.feature_extraction.text import TfidfVectorizer
from scipy.sparse import hstack


def loadDataset(filename):
    dataset = []
    csv_reader = csv.reader(codecs.open('data/' + filename + '.csv', 'rU', 'utf-8'), delimiter=',')
    for row in csv_reader:
        dataset.append(row)
    return dataset



def combineAllTweetsPerUser(dataset):
    users = {}
    for row in dataset:
        TWETINDEX = 5 if len(row) == 7 else 7
        username = row[1]
        if username in users:
            users[username][5] += row[TWETINDEX]
        else:
            users[username] = row[0:5] + [row[5], row[6]]
    return list(users.values())


np.random.seed(0)
data = loadDataset('train')
users = combineAllTweetsPerUser(data)
np.random.shuffle(users)


labels = {'unrelated': 0, 'pro_ed': 1, 'pro_recovery': 0}




y_train = [labels[u[6]] for u in users]
#y_test = [labels[u[6]] for u in users_test]


tweet_corpus = [u[5] for u in users]
#tweet_corpus_test = [u[5] for u in users_test]


def getScores(x, y):
    clf = svm.LinearSVC()
    clf.fit(x, y)
    f1_score = cross_val_score(clf, x, y, cv=5, scoring='f1')
    f1_mean = np.mean(f1_score)
    print('f1', f1_mean)
    precision = cross_val_score(clf, x, y, cv=5, scoring='precision')
    p_mean = np.mean(precision)
    print('precision', p_mean)
    recall = cross_val_score(clf, x, y, cv=5, scoring='recall')
    r_mean = np.mean(recall)
    print('recall', r_mean)
    return p_mean, r_mean, f1_mean


uni_vectorizer = TfidfVectorizer(ngram_range=(1, 1), stop_words='english', max_features=20000)
# Use entire dataset for evaluating the performance
uni_x = uni_vectorizer.fit_transform(tweet_corpus)
uni_x_test = uni_vectorizer.transform(tweet_corpus_test)
print('unigrams')
getScores(uni_x, y_train)


# Bigrams
bi_vectorizer = TfidfVectorizer(ngram_range=(2, 2), stop_words='english', max_features=10000)
bi_x = bi_vectorizer.fit_transform(tweet_corpus)
bi_x_test = bi_vectorizer.transform(tweet_corpus_test)
print('bigrams')
getScores(bi_x, y_train)



tri_vectorizer = TfidfVectorizer(ngram_range=(3, 3), stop_words='english', max_features=10000)
tri_x = tri_vectorizer.fit_transform(tweet_corpus)
tri_x_test = tri_vectorizer.transform(tweet_corpus_test)
print('trigrams')
getScores(tri_x, y_train)


char_vectorizer = TfidfVectorizer(ngram_range=(3, 15), analyzer='char', stop_words='english', max_features=20000)
# Use entire dataset for evaluating the performance
char_x = char_vectorizer.fit_transform(tweet_corpus)
char_x_test = char_vectorizer.transform(tweet_corpus_test)
print('chargrams')
getScores(char_x, y_train)


combined_x = hstack([uni_x, bi_x, tri_x])
getScores(combined_x, y_train)

bi_tri_x = hstack([bi_x, tri_x])
getScores(bi_tri_x, y_train)


def n_gram_test(n, max_features, corpus, labels):
    vectorizer = TfidfVectorizer(ngram_range=(n, n), stop_words='english', max_features=max_features)
    x = vectorizer.fit_transform(corpus)
    return getScores(x, labels)


def create_table(matrix):
    table_string = ''
    for row in matrix:
        for element in row[:-1]:
            table_string += str(element) + ' & '
        table_string += str(row[-1]) + ' \\\\ \n'
    return table_string


latex_table = []

for max_features in range(2000, 5001, 3000):
    print(max_features)
    latex_table.append(n_gram_test(1, max_features, tweet_corpus, y_train))



create_table(latex_table)
# Test all max features for all n
for n in range(1, 4):
    print('n = ', n)
    for max_features in range(300, 1501, 300):
        print(max_features)
        n_gram_test(n, max_features, tweet_corpus, y_train)



"""
bi-grams num features:
300
f1 0.957638581816
precision 0.982894647279
recall 0.933684210526
(0.98289464727948617, 0.93368421052631567, 0.957638581816045)
600
f1 0.964349800416
precision 0.982575747442
recall 0.946842105263
(0.9825757474415141, 0.94684210526315782, 0.96434980041628438)
900
f1 0.963674319492
precision 0.98574303129
recall 0.942631578947
(0.98574303129040408, 0.94263157894736838, 0.96367431949168014)
1200
f1 0.965889386922
precision 0.986293735934
recall 0.946315789474
(0.98629373593422343, 0.94631578947368422, 0.96588938692154291)
1500
f1 0.968597710464
precision 0.987945057816
recall 0.95
(0.98794505781628317, 0.94999999999999996, 0.96859771046370668)
2000
f1 0.970835763094
precision 0.986938430527
recall 0.955263157895
(0.9869384305265354, 0.95526315789473681, 0.97083576309433239)
5000
f1 0.973835881877
precision 0.9875350154
recall 0.960526315789
(0.98753501540046074, 0.96052631578947367, 0.97383588187749692)
8000
f1 0.971348065482
precision 0.987454775429
recall 0.955789473684
(0.98745477542942428, 0.95578947368421052, 0.9713480654819785)
11000
f1 0.971345418052
precision 0.987992723763
recall 0.955263157895
(0.98799272376285407, 0.95526315789473704, 0.97134541805180952)


Unigrams:
300

f1 0.980079644786
precision 0.989280371299
recall 0.971052631579
(0.98928037129925916, 0.97105263157894739, 0.98007964478565923)

600
f1 0.980910106117
precision 0.988806690396
recall 0.973157894737
(0.9888066903957643, 0.97315789473684211, 0.98091010611691321)

900
f1 0.983049782498
precision 0.989887837187
recall 0.976315789474
(0.9898878371865385, 0.97631578947368425, 0.98304978249770991)

1200
f1 0.982515788399
precision 0.989343170899
recall 0.975789473684
(0.98934317089910773, 0.97578947368421054, 0.9825157883985558)

1500
f1 0.983848024839
precision 0.99041977514
recall 0.977368421053
(0.99041977514027069, 0.97736842105263155, 0.98384802483923828)

2000
f1 0.983849367575
precision 0.990432563914
recall 0.977368421053
(0.99043256391444212, 0.97736842105263155, 0.98384936757469532)

5000
f1 0.983847918777
precision 0.990433824508
recall 0.977368421053
(0.9904338245076667, 0.97736842105263155, 0.98384791877747801)

8000
f1 0.983568978367
precision 0.990951674924
recall 0.976315789474
(0.99095167492441671, 0.97631578947368425, 0.98356897836698243)

11000
f1 0.983581654436
precision 0.990431130304
recall 0.976842105263
(0.99043113030424768, 0.97684210526315796, 0.98358165443646395)

14000
f1 0.983313950593
precision 0.990428293425
recall 0.976315789474
(0.99042829342481498, 0.97631578947368425, 0.98331395059260118)

17000
f1 0.982770851745
precision 0.990405658588
recall 0.975263157895
(0.99040565858838703, 0.97526315789473694, 0.98277085174476153)

20000
f1 0.983039265678
precision 0.990408510638
recall 0.975789473684
(0.99040851063829793, 0.97578947368421054, 0.98303926567839583)

"""


