
import csv
import codecs
import numpy as np
from sklearn.model_selection import cross_val_score
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn import svm
from scipy.sparse import hstack
import re


def combineAllTweetsPerUser(dataset):
    users = {}
    for row in dataset:
        if row[-1] == 'class': continue
        TWETINDEX = 5 if len(row) == 7 else 7
        username = row[1]
        if username in users:
            users[username][5] += row[TWETINDEX]
        else:
            if len(row) == 7:
                users[username] = row[0:5] + [row[5], row[6]]
            else:
                users[username] = [row[1], row[5], row[2], row[3], row[6], row[7], row[9]]
    return list(users.values())


def extractEmojis(users_tweets):
    emojis = []
    regex = r"EMOJI[a-zA-Z]+"
    for user in users_tweets:
        emojis.append(' '.join(re.findall(regex, user)))
    return emojis


def loadDataset(filename):
    dataset = []
    csv_reader = csv.reader(codecs.open('data/' + filename + '.csv', 'rU', 'utf-8'), delimiter=',')
    for row in csv_reader:
        dataset.append(row)
    return dataset


def getScores(x, y):
    clf = svm.LinearSVC()
    clf.fit(x, y)
    f1_score = cross_val_score(clf, x, y, cv=5, scoring='f1_weighted')
    f1_mean = np.mean(f1_score)
    print('f1', f1_mean)
    precision = cross_val_score(clf, x, y, cv=5, scoring='precision_weighted')
    p_mean = np.mean(precision)
    print('precision', p_mean)
    recall = cross_val_score(clf, x, y, cv=5, scoring='recall_weighted')
    r_mean = np.mean(recall)
    print('recall', r_mean)
    return round(p_mean, 3), round(r_mean, 3), round(f1_mean, 3)


def create_table(matrix):
    table_string = ''
    for row in matrix:
        for element in row[:-1]:
            table_string += str(element) + ' & '
        table_string += str(row[-1]) + ' \\\\ \n'
    return table_string


np.random.seed(0)
data = loadDataset('train')
users = combineAllTweetsPerUser(data)
del data
np.random.shuffle(users)

labels = {'unrelated': 0, 'pro_ed': 1, 'pro_recovery': 2}
y_train = [labels[u[6]] for u in users]


latex_table = []

# Unigrams
tweet_corpus = [u[5] for u in users]
uni_vectorizer = TfidfVectorizer(ngram_range=(1, 1), stop_words='english', max_features=20000)
uni_x = uni_vectorizer.fit_transform(tweet_corpus)
print('unigrams')
latex_table.append(['Unigrams'] + list(getScores(uni_x, y_train)))


# Bigrams
bi_vectorizer = TfidfVectorizer(ngram_range=(2, 2), stop_words='english', max_features=10000)
bi_x = bi_vectorizer.fit_transform(tweet_corpus)
print('bigrams')
latex_table.append(['Bigrams'] + list(getScores(bi_x, y_train)))


# Username (user-id + displayname)
name_corpus = [u[0] + u[1] for u in users]
name_vectorizer = TfidfVectorizer(analyzer='char', ngram_range=(3, 15), stop_words='english', max_features=10000)
name_x = name_vectorizer.fit_transform(name_corpus)
print('username')
latex_table.append(['Username'] + list(getScores(name_x, y_train)))


# Bio
bio_corpus = [u[3] for u in users]
bio_vectorizer = TfidfVectorizer(ngram_range=(1, 2), stop_words='english', max_features=2000)
bio_x = bio_vectorizer.fit_transform(bio_corpus)
print('bio')
latex_table.append(['Bio'] + list(getScores(bio_x, y_train)))


# Emojis
emoji_corpus = extractEmojis(tweet_corpus)
emoji_vectorizer = TfidfVectorizer(ngram_range=(1, 1), stop_words='english', max_features=20000)
emoji_x = emoji_vectorizer.fit_transform(emoji_corpus)
print('emojis')
latex_table.append(['Emojis'] + list(getScores(emoji_x, y_train)))


# Combined scores
combined_x = hstack([uni_x, bi_x, name_x, bio_x, emoji_x])
print('combined')
latex_table.append(['Combined'] + list(getScores(combined_x, y_train)))


print(create_table(latex_table))


